package de.uka.ipd.algorithms.cfg;

import java.io.PrintStream;
import java.util.HashSet;
import java.util.Set;

/**
 * a control flow graph
 */
public class CFG {
	protected CFGNode start;
	protected CFGNode end;
	protected int     nextIdx = 0;
	
	public CFG() {
		start = createNode("Start");
		end   = createNode("End");
	}
	
	public CFGNode getEnd() {
		return end;
	}
	public CFGNode getStart() {
		return start;
	}
	public CFGNode createNode(String name) {
		CFGNode node = new CFGNode(name, nextIdx);
		++nextIdx;
		
		return node;
	}
	
	private Set<CFGNode> visited;
	private void printNode(PrintStream out, CFGNode node) {
		visited.add(node);
		if(node.getLabel() != node.getName()) {
			out.println("\t" + node.getName() + " [label=\"" + node.getLabel().replace("\n", "\\n") + "\"];\n");
		}
		
		for(CFGNode succ: node.getSuccs()) {			
			out.println("\t" + node.getName() + " -> " + succ.getName() + ";");
			
			if(visited.contains(succ))
				continue;
			printNode(out, succ);			
		}
	}
	
	public void printDotFile(PrintStream out) {
		out.println("digraph G {");
		visited = new HashSet<CFGNode>();
		
		printNode(out, getStart());	
		
		visited = null;
		out.println("}");
	}
	
	public int nodeCount() {
		return nextIdx;
	}
}
